-- DO NOT HOST AS TYPE NETSCRIPT!

Triggers = {}

scale = { 1.0, .75, .50, .45, .40, .333, .20, .10 }

-- Alternate Scale. Slower change.
-- scale = { 1.0, .95, .75, .666, .50, .40, .333, .20, .10 }

-- Overlay shows individual player scales along the top of the screen.
-- Good for debug, but it can be destracting for players.
DISPLAY_OVERLAY = false

-- Superscaler: scale damage based on how far player is ahead or behind
--   or could scaled based on target hit and difference between particular players
--   maybe change speed or other things too. (start with less life, etc)

-- Based on individual difference in score. Find difference between two players.
-- Top player scales by 1 / log(difference ^ 2), buttom player by log(difference ^ 2)
--   ex:  p._superscalar[t] = 1 / math.log(difference * difference * 1.0)
--
-- Log of the squares not an effective scale. Because individual differences
-- in score between players are likely to be small (< 10), the scale function
-- can be expressed explicitly in the form of an index table.

-- .kills[player] : Kill count against player


-- Function should ignore small differences in score between equally skilled
-- players, but change rapidly as the difference grows. Even at 50% damage,
-- good players can be took effective. The rocket presents a particular challenge,
-- because it remains too effective at low scales, while other weapons become
-- useless. Maybe add a special exemption or weapon scale.

function Triggers.init()
	for p in Players() do
		p._superscalar = {}  -- need to initialize a new table for each player
	end
end

function Triggers.idle()

	if(Game.type == "netscript") then
		Players.print("Do not host as netscript. Use EMFH, KOTH, etc.")
	end

	for p in Players() do
		for t in Players() do
			difference = p.kills[t] - t.kills[p]
						
			if difference > 0 then
			
				-- If the difference is greater than the number of entries
				-- in the scale table, then use the last number in the table
				difference = math.min( math.abs(difference), #scale)
				
				p._superscalar[t] = scale[difference]
				--p._superscalar[t] = 1 / math.log(difference * difference * 1.0)
			elseif difference < 0 then 
						difference = math.min( math.abs(difference), #scale)

				p._superscalar[t] = 1 / scale[difference]
				--p._superscalar[t] = math.log(difference * difference * 1.0)
			else -- difference = 0
				p._superscalar[t] = 1.0
			end
		end
	end

	-- Display scales
	if DISPLAY_OVERLAY then showPlayerScales() end

end
	
	

-- Is this working correctly? Who knows. It does something. Good enough.
function Triggers.player_damaged(victim, aggressor, monster, damage_type, amount, projectile)

	if aggressor then
		victim.life = victim.life - ( math.ceil( aggressor._superscalar[victim] * amount ) - amount )
	end

end



function showPlayerScales()
	-- Display scales
	for p in Players() do
		
		x = ""
	
		for s in Players() do
			if not s.local_ then x = x .. string.format("%2s: %.4f ", s.name, p._superscalar[s]) end
		end	
		
		p.overlays[0].text = x
	end
end
